#  Drakkar-Software OctoBot
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
from octobot_commons.constants import CONFIG_ENABLED_OPTION
from octobot_trading.api.modes import init_trading_mode_config
from octobot_trading.constants import CONFIG_EXCHANGES

from octobot.channels.octobot_channel import OctoBotChannelProducer
from octobot_commons.enums import OctoBotChannelSubjects
from octobot_trading.consumers.octobot_channel_consumer import OctoBotChannelTradingActions as TradingActions, \
    OctoBotChannelTradingDataKeys as TradingKeys


class ExchangeProducer(OctoBotChannelProducer):
    def __init__(self, channel, octobot, backtesting, ignore_config=False):
        super().__init__(channel)
        self.octobot = octobot
        self.ignore_config = ignore_config

        self.backtesting = backtesting
        self.exchange_manager_ids = []

    async def start(self):
        init_trading_mode_config(self.octobot.config)
        for exchange_name in self.octobot.config[CONFIG_EXCHANGES]:
            if self.octobot.config[CONFIG_EXCHANGES][exchange_name].get(CONFIG_ENABLED_OPTION, True):
                await self.create_exchange(exchange_name, self.backtesting)

    async def create_exchange(self, exchange_name, backtesting):
        await self.send(bot_id=self.octobot.bot_id,
                        subject=OctoBotChannelSubjects.CREATION.value,
                        action=TradingActions.EXCHANGE.value,
                        data={
                            TradingKeys.TENTACLES_SETUP_CONFIG.value: self.octobot.tentacles_setup_config,
                            TradingKeys.MATRIX_ID.value: self.octobot.evaluator_producer.matrix_id,
                            TradingKeys.BACKTESTING.value: backtesting,
                            TradingKeys.EXCHANGE_CONFIG.value: self.octobot.config,
                            TradingKeys.EXCHANGE_NAME.value: exchange_name,
                        })



#  Drakkar-Software OctoBot
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
import asyncio

import pytest

from octobot_commons.asyncio_tools import ErrorContainer
from octobot_trading.api.exchange import get_exchange_manager_from_exchange_id
from octobot_trading.api.orders import get_open_orders
from octobot_trading.api.profitability import get_profitability_stats
from octobot.api.backtesting import get_independent_backtesting_exchange_manager_ids, stop_independent_backtesting, \
    check_independent_backtesting_remaining_objects
from octobot.backtesting.abstract_backtesting_test import DATA_FILES
from octobot_trading.api.trades import get_trade_history
from tests.test_utils.bot_management import run_independent_backtesting

BACKTESTING_SYMBOLS = ["ICX/BTC", "VEN/BTC", "XRB/BTC", "2020ADA/BTC", "2020ADA/USDT", "BTC/USDT"]

# All test coroutines will be treated as marked.
pytestmark = pytest.mark.asyncio


async def test_single_data_file_backtesting():
    error_container = ErrorContainer()
    asyncio.get_event_loop().set_exception_handler(error_container.exception_handler)
    previous_backtesting = await run_independent_backtesting([DATA_FILES[BACKTESTING_SYMBOLS[0]]])
    current_backtesting = await run_independent_backtesting([DATA_FILES[BACKTESTING_SYMBOLS[0]]])

    await _check_backtesting_results(previous_backtesting, current_backtesting, error_container)


async def test_single_data_file_no_logs_backtesting():
    error_container = ErrorContainer()
    asyncio.get_event_loop().set_exception_handler(error_container.exception_handler)
    previous_backtesting = await run_independent_backtesting([DATA_FILES[BACKTESTING_SYMBOLS[0]]], use_loggers=False)
    current_backtesting = await run_independent_backtesting([DATA_FILES[BACKTESTING_SYMBOLS[0]]], use_loggers=False)

    await _check_backtesting_results(previous_backtesting, current_backtesting, error_container)


async def test_single_data_file_mixed_logs_backtesting():
    error_container = ErrorContainer()
    asyncio.get_event_loop().set_exception_handler(error_container.exception_handler)
    previous_backtesting = await run_independent_backtesting([DATA_FILES[BACKTESTING_SYMBOLS[0]]], use_loggers=False)
    current_backtesting = await run_independent_backtesting([DATA_FILES[BACKTESTING_SYMBOLS[0]]], use_loggers=True)

    await _check_backtesting_results(previous_backtesting, current_backtesting, error_container)


async def test_double_synchronized_data_file_backtesting():
    error_container = ErrorContainer()
    asyncio.get_event_loop().set_exception_handler(error_container.exception_handler)
    previous_backtesting = await run_independent_backtesting([DATA_FILES[BACKTESTING_SYMBOLS[0]],
                                                              DATA_FILES[BACKTESTING_SYMBOLS[1]]],
                                                             timeout=40)
    current_backtesting = await run_independent_backtesting([DATA_FILES[BACKTESTING_SYMBOLS[0]],
                                                             DATA_FILES[BACKTESTING_SYMBOLS[1]]],
                                                            timeout=40)

    await _check_backtesting_results(previous_backtesting, current_backtesting, error_container)


async def test_double_partially_synchronized_data_file_backtesting_common_only():
    error_container = ErrorContainer()
    asyncio.get_event_loop().set_exception_handler(error_container.exception_handler)
    previous_backtesting = await run_independent_backtesting([DATA_FILES[BACKTESTING_SYMBOLS[3]],
                                                              DATA_FILES[BACKTESTING_SYMBOLS[4]]],
                                                             timeout=40)
    current_backtesting = await run_independent_backtesting([DATA_FILES[BACKTESTING_SYMBOLS[3]],
                                                             DATA_FILES[BACKTESTING_SYMBOLS[4]]],
                                                            timeout=40)

    await _check_backtesting_results(previous_backtesting, current_backtesting, error_container)


async def test_double_partially_synchronized_data_file_backtesting_all_data_files_range():
    error_container = ErrorContainer()
    asyncio.get_event_loop().set_exception_handler(error_container.exception_handler)
    previous_backtesting = await run_independent_backtesting([DATA_FILES[BACKTESTING_SYMBOLS[3]],
                                                              DATA_FILES[BACKTESTING_SYMBOLS[4]]],
                                                             run_on_common_part_only=False,
                                                             timeout=40)
    current_backtesting = await run_independent_backtesting([DATA_FILES[BACKTESTING_SYMBOLS[3]],
                                                             DATA_FILES[BACKTESTING_SYMBOLS[4]]],
                                                            run_on_common_part_only=False,
                                                            timeout=40)

    await _check_backtesting_results(previous_backtesting, current_backtesting, error_container)


async def test_double_data_file_3_months_gap_backtesting_all_data_files_range():
    error_container = ErrorContainer()
    asyncio.get_event_loop().set_exception_handler(error_container.exception_handler)
    previous_backtesting = await run_independent_backtesting([DATA_FILES[BACKTESTING_SYMBOLS[5]],
                                                              DATA_FILES[BACKTESTING_SYMBOLS[0]]],
                                                             timeout=80,
                                                             run_on_common_part_only=False)
    current_backtesting = await run_independent_backtesting([DATA_FILES[BACKTESTING_SYMBOLS[5]],
                                                             DATA_FILES[BACKTESTING_SYMBOLS[0]]],
                                                            timeout=80,
                                                            run_on_common_part_only=False)

    await _check_backtesting_results(previous_backtesting, current_backtesting, error_container)


async def _check_backtesting_results(backtesting1, backtesting2, error_container):
    exchange_manager_1 = get_exchange_manager_from_exchange_id(
        get_independent_backtesting_exchange_manager_ids(backtesting1)[0])
    exchange_manager_2 = get_exchange_manager_from_exchange_id(
        get_independent_backtesting_exchange_manager_ids(backtesting2)[0])

    _, previous_profitability, previous_market_profitability, _, _ = get_profitability_stats(exchange_manager_1)
    _, current_profitability, current_market_profitability, _, _ = get_profitability_stats(exchange_manager_2)

    trades = get_trade_history(exchange_manager_1)
    open_orders = get_open_orders(exchange_manager_1)
    # ensure at least one order is either open or got filled
    assert trades + open_orders

    trades = open_orders = exchange_manager_1 = exchange_manager_2 = None  # prevent memory leak

    # ensure no randomness in backtesting
    assert previous_profitability == current_profitability
    assert current_profitability != 0
    assert previous_profitability != 0
    assert previous_market_profitability == current_market_profitability

    await stop_independent_backtesting(backtesting1, memory_check=True)
    await stop_independent_backtesting(backtesting2, memory_check=True)
    asyncio.get_event_loop().call_soon(check_independent_backtesting_remaining_objects, backtesting1)
    asyncio.get_event_loop().call_soon(check_independent_backtesting_remaining_objects, backtesting2)
    await asyncio.create_task(error_container.check())



# cython: language_level=3
#  Drakkar-Software OctoBot
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.


from octobot.backtesting.octobot_backtesting cimport OctoBotBacktesting

cdef class IndependentBacktesting:
    cdef list forced_time_frames

    cdef public dict octobot_origin_config
    cdef public dict backtesting_config
    cdef public object tentacles_setup_config
    cdef public list backtesting_files

    cdef object logger

    cdef public str data_file_path
    cdef public dict symbols_to_create_exchange_classes
    cdef public double risk
    cdef public dict starting_portfolio
    cdef public dict fees_config
    cdef public bint stopped

    cdef public OctoBotBacktesting octobot_backtesting

    cpdef bint is_in_progress(self)
    cpdef double get_progress(self)
    cpdef void log_report(self)

    cdef void _init_default_config_values(self)
    cdef dict _get_exchanges_report(self, str reference_market, object trading_mode)
    cdef void _log_trades_history(self, object exchange_manager, str exchange_name)
    cdef void _log_symbol_report(self, str symbol, object exchange_manager, object min_time_frame)
    cdef void _log_global_report(self, object exchange_manager)
    cdef void _adapt_config(self)
    cdef str _find_reference_market(self)
    cdef void _add_config_default_backtesting_values(self)
    cdef void _add_crypto_currencies_config(self)



#  Drakkar-Software OctoBot
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
from octobot.channels.octobot_channel import OctoBotChannelProducer
from octobot.constants import CONFIG_KEY
from octobot_backtesting.api.backtesting import is_backtesting_enabled
from octobot_services.api.service_feeds import create_service_feed_factory, start_service_feed, stop_service_feed


class ServiceFeedProducer(OctoBotChannelProducer):
    """EvaluatorFactory class:
    - Create service feeds
    """

    def __init__(self, channel, octobot):
        super().__init__(channel)
        self.octobot = octobot

        self.service_feeds = []

    async def start(self):
        in_backtesting = is_backtesting_enabled(self.octobot.config)
        service_feed_factory = create_service_feed_factory(self.octobot.config,
                                                           self.octobot.async_loop,
                                                           self.octobot.bot_id)
        self.service_feeds = [await self.create_feed(service_feed_factory, feed)
                              for feed in service_feed_factory.get_available_service_feeds(in_backtesting)]

    async def create_feed(self, service_feed_factory, feed):
        service_feed = service_feed_factory.create_service_feed(feed)
        if not await start_service_feed(service_feed, False, self.octobot.get_edited_config(CONFIG_KEY)):
            self.logger.error(f"Failed to start {service_feed.get_name()}. Evaluators requiring this service feed "
                              f"might not work properly")
        return service_feed

    async def stop(self):
        for service_feed in self.service_feeds:
            await stop_service_feed(service_feed)



#  Drakkar-Software OctoBot
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.

import os

import aiohttp
import sys
import asyncio
import signal
from threading import Thread
from concurrent.futures import CancelledError

from octobot_tentacles_manager.api.installer import install_all_tentacles

from octobot.constants import DEFAULT_TENTACLES_URL
from octobot import get_bot, set_bot
from octobot_commons.config_util import encrypt
from octobot_tentacles_manager.api.configurator import get_tentacles_setup_config, ensure_setup_configuration
from octobot_commons.logging.logging_util import get_logger
from octobot_tentacles_manager.api.loader import reload_tentacle_info
from octobot_tentacles_manager.cli import handle_tentacles_manager_command
from octobot.logger import init_logger

COMMANDS_LOGGER_NAME = "Commands"


def call_tentacles_manager(command_args):
    init_logger()
    sys.exit(handle_tentacles_manager_command(command_args, tentacles_url=DEFAULT_TENTACLES_URL))


def exchange_keys_encrypter(catch=False):
    try:
        api_key_crypted = encrypt(input("ENTER YOUR API-KEY : ")).decode()
        api_secret_crypted = encrypt(input("ENTER YOUR API-SECRET : ")).decode()
        print(f"Here are your encrypted exchanges keys : \n "
              f"\t- API-KEY : {api_key_crypted}\n"
              f"\t- API-SECRET : {api_secret_crypted}\n\n"
              f"Your new exchange key configuration is : \n"
              f'\t"api-key": "{api_key_crypted}",\n'
              f'\t"api-secret": "{api_secret_crypted}"\n')
    except Exception as e:
        if not catch:
            get_logger(COMMANDS_LOGGER_NAME).error(f"Fail to encrypt your exchange keys, please try again ({e}).")
            raise e


def start_strategy_optimizer(config, commands):
    from octobot.api.strategy_optimizer import create_strategy_optimizer, \
        get_optimizer_is_properly_initialized, find_optimal_configuration, print_optimizer_report
    tentacles_setup_config = get_tentacles_setup_config()
    optimizer = create_strategy_optimizer(config, tentacles_setup_config, commands[0])
    if get_optimizer_is_properly_initialized(optimizer):
        find_optimal_configuration(optimizer)
        print_optimizer_report(optimizer)


def run_tentacles_installation():
    asyncio.run(_install_all_tentacles())


async def _install_all_tentacles():
    async with aiohttp.ClientSession() as aiohttp_session:
        await install_all_tentacles(DEFAULT_TENTACLES_URL, aiohttp_session=aiohttp_session)


def _signal_handler(_, __):
    # run Commands.BOT.stop_threads in thread because can't use the current asyncio loop
    stopping_thread = Thread(target=get_bot().task_manager.stop_tasks(), name="Commands signal_handler stop_tasks")
    stopping_thread.start()
    stopping_thread.join()
    os._exit(0)


def run_bot(bot, logger):
    # handle CTRL+C signal
    signal.signal(signal.SIGINT, _signal_handler)

    # start bot
    bot.task_manager.run_forever(start_bot(bot, logger))


async def start_bot(bot, logger, catch=False):
    try:
        # load tentacles details
        reload_tentacle_info()
        # ensure tentacles config exists or create a new one
        await ensure_setup_configuration()

        # start
        try:
            set_bot(bot)
            await bot.initialize()
        except CancelledError:
            logger.info("Core engine tasks cancelled.")

    except Exception as e:
        logger.exception(e, True, f"OctoBot Exception : {e}")
        if not catch:
            raise e
        stop_bot(bot)


def stop_bot(bot, force=False):
    bot.task_manager.stop_tasks()
    if force:
        os._exit(0)


def restart_bot():
    if sys.argv[0].endswith(".py"):
        os.execl(sys.executable, sys.executable, *sys.argv)
    else:
        # prevent binary to add self as first argument
        os.execl(sys.executable, *sys.argv)



#  Drakkar-Software OctoBot
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.

import time
import asyncio
import json
import requests
import threading
from concurrent.futures import CancelledError

from octobot_commons.constants import CONFIG_METRICS_BOT_ID, METRICS_URL, METRICS_ROUTE_GEN_BOT_ID, \
    METRICS_ROUTE_UPTIME, METRICS_ROUTE_REGISTER, TIMER_BEFORE_METRICS_REGISTRATION_SECONDS, \
    TIMER_BETWEEN_METRICS_UPTIME_UPDATE, CONFIG_METRICS, \
    CONFIG_ENABLED_OPTION
from octobot_evaluators.api.evaluators import get_evaluator_classes_from_type
from octobot_evaluators.enums import EvaluatorMatrixTypes
from octobot_trading.api.exchange import get_trading_pairs, get_exchange_names, get_exchange_managers_from_exchange_ids
from octobot.community.community_fields import CommunityFields
from octobot_services.constants import CONFIG_CATEGORY_NOTIFICATION, CONFIG_NOTIFICATION_TYPE
from octobot_commons.logging.logging_util import get_logger
from octobot_commons.config_manager import get_metrics_enabled, simple_save_config_update
from octobot_commons.os_util import get_current_platform, get_octobot_type

from octobot_trading.api.profitability import get_reference_market, get_current_portfolio_value, \
    get_origin_portfolio_value, get_profitability_stats
from octobot_trading.api.trader import is_trader_enabled_in_config, is_trader_simulator_enabled_in_config


class CommunityManager:
    _headers = {'Content-type': 'application/json', 'Accept': 'text/plain'}

    def __init__(self, octobot_api):
        self.octobot_api = octobot_api
        self.bot_config = octobot_api.get_global_config()
        self.edited_config = octobot_api.get_edited_config()
        self.enabled = get_metrics_enabled(self.edited_config)
        self.bot_id = self._init_config_bot_id(self.edited_config)
        self.reference_market = get_reference_market(self.edited_config)
        self.logger = get_logger(self.__class__.__name__)
        self.current_config = None
        self.keep_running = True
        self.session = octobot_api.get_aiohttp_session()
        self.has_real_trader = is_trader_enabled_in_config(self.edited_config)
        self.has_simulator = is_trader_simulator_enabled_in_config(self.edited_config)
        self.exchange_managers = get_exchange_managers_from_exchange_ids(
            self.octobot_api.get_exchange_manager_ids())

    async def start_community_task(self):
        if self.enabled:
            try:
                # first ensure this session is not just a configuration test: register after a timer
                await asyncio.sleep(TIMER_BEFORE_METRICS_REGISTRATION_SECONDS)
                await self.register_session()
                while self.keep_running:
                    # send a keepalive at periodic intervals
                    await asyncio.sleep(TIMER_BETWEEN_METRICS_UPTIME_UPDATE)
                    try:
                        await self._update_uptime_and_profitability()
                    except Exception as e:
                        self.logger.debug(f"Exception when handling community data : {e}")
            except CancelledError:
                pass
            except Exception as e:
                self.logger.debug(f"Exception when handling community registration: {e}")

    async def stop_task(self):
        self.keep_running = False
        await self.session.close()

    @staticmethod
    def should_register_bot(config):
        existing_id = CommunityManager._init_config_bot_id(config)
        return not existing_id

    @staticmethod
    def background_get_id_and_register_bot(octobot_api):
        community_manager = CommunityManager(octobot_api)
        threading.Thread(target=community_manager._blocking_get_id_and_register).start()

    def _blocking_get_id_and_register(self):
        try:
            resp = requests.get(f"{METRICS_URL}{METRICS_ROUTE_GEN_BOT_ID}", headers=self._headers)
            text = resp.text
            if resp.status_code != 200:
                self.logger.debug(f"Impossible to get bot id: status code: {resp.status_code}, text: {text}")
            else:
                self.bot_id = json.loads(text)
                self._save_bot_id()
                community = self._get_bot_community()
                requests.post(f"{METRICS_URL}{METRICS_ROUTE_REGISTER}", json=community, headers=self._headers)
        except Exception as e:
            self.logger.debug(f"Error when handling community: {e}")

    @staticmethod
    def _init_config_bot_id(config):
        if CONFIG_METRICS in config and config[CONFIG_METRICS] and \
                CONFIG_METRICS_BOT_ID in config[CONFIG_METRICS]:
            return config[CONFIG_METRICS][CONFIG_METRICS_BOT_ID]
        else:
            return None

    async def register_session(self, retry_on_error=True):
        self.current_config = await self._get_current_community_config()
        await self._post_community_data(METRICS_ROUTE_REGISTER, self.current_config, retry_on_error)

    async def _update_uptime_and_profitability(self, retry_on_error=True):
        self.current_config[CommunityFields.CURRENT_SESSION.value][CommunityFields.UP_TIME.value] = \
            int(time.time() - self.octobot_api.get_start_time())
        self.current_config[CommunityFields.CURRENT_SESSION.value][CommunityFields.PROFITABILITY.value] = \
            self._get_profitability()
        await self._post_community_data(METRICS_ROUTE_UPTIME, self.current_config, retry_on_error)

    async def _get_current_community_config(self):
        if not self.bot_id:
            await self._init_bot_id()
        if self.bot_id:
            return self._get_bot_community()

    def _get_bot_community(self):
        return {
            CommunityFields.ID.value: self.bot_id,
            CommunityFields.CURRENT_SESSION.value: {
                CommunityFields.STARTED_AT.value: int(self.octobot_api.get_start_time()),
                CommunityFields.UP_TIME.value: int(time.time() - self.octobot_api.get_start_time()),
                CommunityFields.SIMULATOR.value: self.has_simulator,
                CommunityFields.TRADER.value: self.has_real_trader,
                CommunityFields.EVAL_CONFIG.value: self._get_eval_config(),
                CommunityFields.PAIRS.value: self._get_traded_pairs(),
                CommunityFields.EXCHANGES.value: list(get_exchange_names()),
                CommunityFields.NOTIFICATIONS.value: self._get_notification_types(),
                CommunityFields.TYPE.value: get_octobot_type(),
                CommunityFields.PLATFORM.value: get_current_platform(),
                CommunityFields.REFERENCE_MARKET.value: self.reference_market,
                CommunityFields.PORTFOLIO_VALUE.value: self._get_real_portfolio_value(),
                CommunityFields.PROFITABILITY.value: self._get_profitability()
            }
        }

    def _get_profitability(self):
        total_origin_values = 0
        total_profitability = 0

        for exchange_manager in self.exchange_managers:
            profitability, _, _, _, _ = get_profitability_stats(exchange_manager)
            total_profitability += profitability
            total_origin_values += get_current_portfolio_value(exchange_manager)

        return total_profitability * 100 / total_origin_values if total_origin_values > 0 else 0

    def _get_real_portfolio_value(self):
        if self.has_real_trader:
            total_value = 0
            for exchange_manager in self.exchange_managers:
                current_value = get_current_portfolio_value(exchange_manager)
                # current_value might be 0 if no trades have been made / canceled => use origin value
                if current_value == 0:
                    current_value = get_origin_portfolio_value(exchange_manager)
                total_value += current_value
            return total_value
        else:
            return 0

    def _get_traded_pairs(self):
        pairs = set()
        for exchange_manager in self.exchange_managers:
            pairs = pairs.union(get_trading_pairs(exchange_manager))
        return list(pairs)

    def _get_notification_types(self):
        has_notifications = CONFIG_CATEGORY_NOTIFICATION in self.edited_config \
                            and CONFIG_NOTIFICATION_TYPE in self.edited_config[CONFIG_CATEGORY_NOTIFICATION]
        return self.edited_config[CONFIG_CATEGORY_NOTIFICATION][CONFIG_NOTIFICATION_TYPE] if has_notifications else []

    def _get_eval_config(self):
        tentacle_setup_config = self.octobot_api.get_tentacles_setup_config()
        # trading mode
        config_eval = [self.octobot_api.get_trading_mode().get_name()]

        # strategies
        for strategy in get_evaluator_classes_from_type(EvaluatorMatrixTypes.STRATEGIES.value,
                                                        self.bot_config,
                                                        tentacle_setup_config):
            config_eval.append(strategy.get_name())

        # evaluators
        evaluators = get_evaluator_classes_from_type(EvaluatorMatrixTypes.TA.value,
                                                     self.bot_config,
                                                     tentacle_setup_config)
        evaluators += get_evaluator_classes_from_type(EvaluatorMatrixTypes.SOCIAL.value,
                                                      self.bot_config,
                                                      tentacle_setup_config)
        evaluators += get_evaluator_classes_from_type(EvaluatorMatrixTypes.REAL_TIME.value,
                                                      self.bot_config,
                                                      tentacle_setup_config)
        for evaluator in evaluators:
            config_eval.append(evaluator.get_name())
        return config_eval

    async def _init_bot_id(self):
        try:
            async with self.session.get(f"{METRICS_URL}{METRICS_ROUTE_GEN_BOT_ID}", headers=self._headers) as resp:
                text = await resp.text()
                if resp.status != 200:
                    self.logger.debug(f"Impossible to get bot id: status code: {resp.status}, text: {text}")
                else:
                    self.bot_id = json.loads(text)
                    self._save_bot_id()
        except Exception as e:
            self.logger.debug(f"Error when handling community data : {e}")

    def _save_bot_id(self):
        if CONFIG_METRICS not in self.edited_config or not self.edited_config[CONFIG_METRICS]:
            self.edited_config[CONFIG_METRICS] = {CONFIG_ENABLED_OPTION: True}
        self.edited_config[CONFIG_METRICS][CONFIG_METRICS_BOT_ID] = self.bot_id
        simple_save_config_update(self.edited_config)

    async def _post_community_data(self, route, bot, retry_on_error):
        try:
            async with self.session.post(f"{METRICS_URL}{route}", json=bot, headers=self._headers) as resp:
                await self._handle_post_error(resp, retry_on_error)
        except Exception as e:
            self.logger.debug(f"Error when handling community data : {e}")

    async def _handle_post_error(self, resp, retry_on_error):
        if resp.status != 200:
            if resp.status == 404:
                # did not found bot with id in config: generate new id and register new bot
                if retry_on_error:
                    await self._init_bot_id()
                    await self.register_session(retry_on_error=False)
            else:
                self.logger.debug(f"Impossible to send community data : "
                                  f"status code: {resp.status}, "
                                  f"text: {await resp.text()}")



#  Drakkar-Software OctoBot-Trading
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
from octobot.channels.octobot_channel import OctoBotChannel
from octobot.logger import init_exchange_chan_logger
from octobot_channels.channels.channel_instances import set_chan_at_id
from octobot_channels.util.channel_creator import create_channel_instance
from octobot_commons.enums import OctoBotChannelSubjects
from octobot_commons.logging.logging_util import get_logger
from octobot_trading.api.exchange import get_exchange_configuration_from_exchange_id
from octobot_trading.consumers.octobot_channel_consumer import OctoBotChannelTradingActions as TradingActions, \
    OctoBotChannelTradingDataKeys as TradingKeys, octobot_channel_callback as octobot_channel_trading_callback
from octobot_evaluators.consumers.octobot_channel_consumer import OctoBotChannelEvaluatorActions as EvaluatorActions, \
    octobot_channel_callback as octobot_channel_evaluator_callback


class OctoBotChannelGlobalConsumer:

    def __init__(self, octobot):
        self.octobot = octobot
        self.logger = get_logger(self.__class__.__name__)

        # the list of octobot channel consumers
        self.octobot_channel_consumers = []

        # the OctoBot Channel instance
        self.octobot_channel = None

    async def initialize(self):
        # Creates OctoBot Channel
        self.octobot_channel = await create_channel_instance(OctoBotChannel, set_chan_at_id, is_synchronized=True,
                                                             bot_id=self.octobot.bot_id)

        # Initialize global consumer
        self.octobot_channel_consumers.append(
            await self.octobot_channel.new_consumer(self.octobot_channel_callback, bot_id=self.octobot.bot_id))

        # Initialize trading consumer
        self.octobot_channel_consumers.append(
            await self.octobot_channel.new_consumer(
                octobot_channel_trading_callback,
                bot_id=self.octobot.bot_id,
                action=[action.value for action in TradingActions]
            ))

        # Initialize evaluator consumer
        self.octobot_channel_consumers.append(
            await self.octobot_channel.new_consumer(
                octobot_channel_evaluator_callback,
                bot_id=self.octobot.bot_id,
                action=[action.value for action in EvaluatorActions]
            ))

    async def octobot_channel_callback(self, bot_id, subject, action, data) -> None:
        """
        OctoBot channel consumer callback
        :param bot_id: the callback bot id
        :param subject: the callback subject
        :param action: the callback action
        :param data: the callback data
        """
        if subject == OctoBotChannelSubjects.NOTIFICATION.value:
            if action == TradingActions.EXCHANGE.value:
                if TradingKeys.EXCHANGE_ID.value in data:
                    exchange_id = data[TradingKeys.EXCHANGE_ID.value]
                    self.octobot.exchange_producer.exchange_manager_ids.append(exchange_id)
                    await init_exchange_chan_logger(exchange_id)
                    exchange_configuration = get_exchange_configuration_from_exchange_id(exchange_id)
                    await self.octobot.evaluator_producer.create_evaluators(exchange_configuration)
            if action == EvaluatorActions.EVALUATOR.value:
                pass

    async def stop(self) -> None:
        """
        Remove all OctoBot Channel consumers
        """
        for consumer in self.octobot_channel_consumers:
            await self.octobot_channel.remove_consumer(consumer)



#  Drakkar-Software OctoBot
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.

from __future__ import print_function
from distutils.version import LooseVersion
import os
import sys

MIN_TENTACLE_MANAGER_VERSION = "1.0.10"

# check compatible tentacle manager
try:
    from octobot_tentacles_manager import VERSION
    if LooseVersion(VERSION) < MIN_TENTACLE_MANAGER_VERSION:
        print("OctoBot requires OctoBot-Tentacles-Manager in a minimum version of " + MIN_TENTACLE_MANAGER_VERSION +
              " you can install and update OctoBot-Tentacles-Manager using the following command: "
              "python3 -m pip install -U OctoBot-Tentacles-Manager", file=sys.stderr)
        sys.exit(-1)
except ImportError:
    print("OctoBot requires OctoBot-Tentacles-Manager, you can install it using "
          "python3 -m pip install -U OctoBot-Tentacles-Manager", file=sys.stderr)
    sys.exit(-1)

# binary tentacle importation
sys.path.append(os.path.dirname(sys.executable))

# if compatible version, can proceed with imports
from octobot_commons.enums import PlatformsName
from octobot_commons.os_util import get_os
from octobot_commons.logging.logging_util import get_logger

# check sudo rights
if get_os() is not PlatformsName.WINDOWS and os.getuid() == 0:
    get_logger("PythonChecker").warning("OctoBot is started with admin / sudo rights that are not required, "
                                        "please check you starting command ")

bot_instance = None
global_config = None


def __init__(bot, config):
    global bot_instance
    bot_instance = bot

    global global_config
    global_config = config


# TODO: find a better way to keep track of the bot instance in octobot module
def set_bot(bot):
    global bot_instance
    bot_instance = bot


def get_bot():
    return bot_instance


def get_config():
    return global_config



# cython: language_level=3
#  Drakkar-Software OctoBot
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.

from octobot.backtesting.abstract_backtesting_test cimport AbstractBacktestingTest
from octobot.strategy_optimizer.test_suite_result cimport TestSuiteResult

cdef class StrategyTestSuite(AbstractBacktestingTest):
    cdef list _profitability_results
    cdef list _trades_counts

    cdef public double current_progress
    cdef public list exceptions
    cdef public list evaluators

    cpdef TestSuiteResult get_test_suite_result(self)

    cdef void _handle_results(self, object independent_backtesting, object profitability)



#  Drakkar-Software OctoBot
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
from octobot.strategy_optimizer.strategy_optimizer import StrategyOptimizer


def create_strategy_optimizer(config, tentacles_setup_config, strategy_name) -> StrategyOptimizer:
    return StrategyOptimizer(config, tentacles_setup_config, strategy_name)


def find_optimal_configuration(strategy_optimizer, TAs=None, time_frames=None, risks=None) -> None:
    strategy_optimizer.find_optimal_configuration(TAs=TAs, time_frames=time_frames, risks=risks)


def print_optimizer_report(strategy_optimizer) -> None:
    strategy_optimizer.print_report()


def get_optimizer_report(strategy_optimizer) -> list:
    return strategy_optimizer.get_report()


def get_optimizer_results(strategy_optimizer) -> list:
    return strategy_optimizer.run_results


def get_optimizer_overall_progress(strategy_optimizer) -> int:
    return strategy_optimizer.get_overall_progress()


def is_optimizer_in_progress(strategy_optimizer) -> bool:
    return strategy_optimizer.is_in_progress()


def is_optimizer_computing(strategy_optimizer) -> bool:
    return strategy_optimizer.is_computing


def get_optimizer_errors_description(strategy_optimizer) -> str:
    return strategy_optimizer.get_errors_description()


def get_optimizer_current_test_suite_progress(strategy_optimizer) -> int:
    return strategy_optimizer.get_current_test_suite_progress()


def get_optimizer_strategy(strategy_optimizer) -> object:
    return strategy_optimizer.strategy_class


def get_optimizer_all_time_frames(strategy_optimizer) -> list:
    return strategy_optimizer.all_time_frames


def get_optimizer_all_TAs(strategy_optimizer) -> list:
    return strategy_optimizer.all_TAs


def get_optimizer_all_risks(strategy_optimizer) -> list:
    return strategy_optimizer.risks


def get_optimizer_trading_mode(strategy_optimizer) -> object:
    return strategy_optimizer.trading_mode


def get_optimizer_is_properly_initialized(strategy_optimizer) -> bool:
    return strategy_optimizer.is_properly_initialized



# cython: language_level=3
#  Drakkar-Software OctoBot
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.


cdef class ConfigurationManager:
    cdef dict configuration_elements

    cpdef void add_element(self, str key, object element)
    cpdef object get_edited_config(self, str key)
    cpdef object get_startup_config(self, str key)

cdef class ConfigurationElement:
    cdef public object config
    cdef public object startup_config
    cdef public object edited_config

cpdef config_health_check(dict config)



# cython: language_level=3
#  Drakkar-Software OctoBot
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.


cdef class OctoBotBacktesting:
    cdef public dict backtesting_config
    cdef public object tentacles_setup_config

    cdef object logger

    cdef public str matrix_id
    cdef public str bot_id
    cdef public list exchange_manager_ids
    cdef public dict symbols_to_create_exchange_classes
    cdef public list evaluators
    cdef public list service_feeds
    cdef public dict fees_config
    cdef public list backtesting_files
    cdef public object backtesting
    cdef public bint run_on_common_part_only

    cpdef void memory_leak_checkup(self, list to_check_elements)
    cpdef void check_remaining_objects(self)

cdef str _get_remaining_object_error(object obj, int expected, tuple actual)



#  Drakkar-Software OctoBot
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
import argparse
import os
import sys

from octobot_commons.os_util import get_current_platform, get_octobot_type
from octobot_tentacles_manager.api.loader import load_tentacles
from octobot_tentacles_manager.cli import register_tentacles_manager_arguments

from octobot.commands import exchange_keys_encrypter, start_strategy_optimizer, \
    run_tentacles_installation, run_bot, call_tentacles_manager
from octobot.configuration_manager import config_health_check
from octobot.constants import LONG_VERSION
from octobot.disclaimer import DISCLAIMER
from octobot.logger import init_logger
from octobot_backtesting.constants import CONFIG_BACKTESTING_DATA_FILES
from octobot_commons.config import load_config, is_config_empty_or_missing, init_config
from octobot_commons.config_manager import validate_config_file, accepted_terms
from octobot_commons.constants import CONFIG_ENABLED_OPTION, CONFIG_FILE, DEFAULT_CONFIG_FILE
from octobot_commons.errors import ConfigError, ConfigTradingError, ConfigEvaluatorError
from octobot_services.api.interfaces import disable_interfaces
from octobot_trading.constants import CONFIG_TRADER, CONFIG_SIMULATOR, CONFIG_TRADING, CONFIG_TRADER_RISK


def update_config_with_args(starting_args, config, logger):
    if starting_args.backtesting:
        try:
            from octobot_backtesting.constants import CONFIG_BACKTESTING, CONFIG_ANALYSIS_ENABLED_OPTION
            if starting_args.backtesting_files:
                config[CONFIG_BACKTESTING][CONFIG_BACKTESTING_DATA_FILES] = starting_args.backtesting_files
            config[CONFIG_BACKTESTING][CONFIG_ENABLED_OPTION] = True
        except ImportError as e:
            logger.error("Can't start backtesting without the octobot_backtesting package properly installed.")
            raise e
        config[CONFIG_TRADER][CONFIG_ENABLED_OPTION] = False
        config[CONFIG_SIMULATOR][CONFIG_ENABLED_OPTION] = True

    if starting_args.simulate:
        config[CONFIG_TRADER][CONFIG_ENABLED_OPTION] = False
        config[CONFIG_SIMULATOR][CONFIG_ENABLED_OPTION] = True

    if starting_args.risk is not None and 0 < starting_args.risk <= 1:
        config[CONFIG_TRADING][CONFIG_TRADER_RISK] = starting_args.risk


# def _check_public_announcements(logger):
#     try:
#         announcement = get_external_resource(EXTERNAL_RESOURCE_PUBLIC_ANNOUNCEMENTS)
#         if announcement:
#             logger.info(announcement)
#     except Exception as e:
#         logger.warning("Impossible to check announcements: {0}".format(e))


def _log_terms_if_unaccepted(config, logger):
    if not accepted_terms(config):
        logger.info("*** Disclaimer ***")
        for line in DISCLAIMER:
            logger.info(line)
        logger.info("... Disclaimer ...")
    else:
        logger.info("Disclaimer accepted by user.")


def _disable_interface_from_param(interface_identifier, param_value, logger):
    if param_value:
        if disable_interfaces(interface_identifier) == 0:
            logger.warning("No " + interface_identifier + " interface to disable")
        else:
            logger.info(interface_identifier.capitalize() + " interface disabled")


def start_octobot(args):
    logger = None
    try:
        if args.version:
            print(LONG_VERSION)
            return

        logger = init_logger()

        # Version
        logger.info("Version : {0}".format(LONG_VERSION))

        # Current running environment
        try:
            logger.debug(f"Running on {get_current_platform()} with {get_octobot_type()}")
        except Exception as e:
            logger.error(f"Impossible to identify the current running environment: {e}")

        # _check_public_announcements(logger)

        logger.info("Loading config files...")

        # configuration loading
        config = load_config(error=False, fill_missing_fields=True)

        if config is None and is_config_empty_or_missing():
            logger.info("No configuration found creating default...")
            init_config()
            config = load_config(error=False)
        else:
            is_valid, e = validate_config_file(config=config)
            if not is_valid:
                logger.error("OctoBot can't repair your config.json file: invalid format: " + str(e))
                raise ConfigError
            config_health_check(config)

        if config is None:
            raise ConfigError

        # Handle utility methods before bot initializing if possible
        if args.encrypter:
            exchange_keys_encrypter()
            return

        # Add tentacles folder to Python path
        sys.path.append(os.path.realpath(os.getcwd()))

        if not load_tentacles(verbose=True):
            logger.info("No tentacles found. Installing default tentacles ...")
            run_tentacles_installation()
            # reload tentacles
            load_tentacles(verbose=True)

        if args.strategy_optimizer:
            start_strategy_optimizer(config, args.strategy_optimizer)
            return

        # In those cases load OctoBot
        from octobot.octobot import OctoBot

        _disable_interface_from_param("telegram", args.no_telegram, logger)
        _disable_interface_from_param("web", args.no_web, logger)

        update_config_with_args(args, config, logger)

        if args.backtesting:
            from octobot.backtesting.octobot_backtesting_factory import OctoBotBacktestingFactory
            bot = OctoBotBacktestingFactory(config, run_on_common_part_only=not args.whole_data_range)
        else:
            bot = OctoBot(config, reset_trading_history=args.reset_trading_history)

        _log_terms_if_unaccepted(config, logger)

        run_bot(bot, logger)

    except ConfigError:
        logger.error("OctoBot can't start without " + CONFIG_FILE + " configuration file." + "\nYou can use " +
                     DEFAULT_CONFIG_FILE + " as an example to fix it.")
        os._exit(-1)

    except ModuleNotFoundError as e:
        if 'tentacles' in str(e):
            logger.error("Impossible to start OctoBot, tentacles are missing.\nTo install tentacles, "
                         "please use the following command:\nstart.py tentacles --install --all")
        else:
            logger.exception(e)
        os._exit(-1)

    except ConfigEvaluatorError:
        logger.error("OctoBot can't start without a valid  configuration file.\n"
                     "This file is generated on tentacle "
                     "installation using the following command:\nstart.py tentacles --install --all")
        os._exit(-1)

    except ConfigTradingError:
        logger.error("OctoBot can't start without a valid configuration file.\n"
                     "This file is generated on tentacle "
                     "installation using the following command:\nstart.py tentacles --install --all")
        os._exit(-1)


def octobot_parser(parser):
    parser.add_argument('-v', '--version', help='Show OctoBot current version.',
                        action='store_true')
    parser.add_argument('-s', '--simulate', help='Force OctoBot to start with the trader simulator only.',
                        action='store_true')
    parser.add_argument('-rts', '--reset-trading-history', help='Force the traders to reset their history. They will '
                                                                'now take the next portfolio as a reference for '
                                                                'profitability and trading simulators will use a '
                                                                'fresh new portfolio.',
                        action='store_true')
    parser.add_argument('-b', '--backtesting', help='Start OctoBot in backesting mode using the backtesting '
                                                    'config stored in config.json.',
                        action='store_true')
    parser.add_argument('-bf', '--backtesting-files', type=str, nargs='+',
                        help='Backtesting files to use (should be provided with -b or --backtesting).',
                        required=False)
    parser.add_argument('-wdr', '--whole-data-range',
                        help='On multiple files backtesting: run on the whole available data instead of the '
                             'common part only (default behavior).',
                        action='store_true')
    parser.add_argument('-r', '--risk', type=float, help='Force a specific risk configuration (between 0 and 1).')
    parser.add_argument('-nw', '--no_web', help="Don't start OctoBot web interface.",
                        action='store_true')
    parser.add_argument('-no', '--no_open_web', help="Don't automatically open web interface.",
                        action='store_true')
    parser.add_argument('-nt', '--no-telegram', help='Start OctoBot without telegram interface, even if telegram '
                                                     'credentials are in config. With this parameter, your Octobot '
                                                     'won`t reply to any telegram command but is still able to listen '
                                                     'to telegram feed and send telegram notifications',
                        action='store_true')
    parser.add_argument('--encrypter', help="Start the exchange api keys encrypter. This tool is useful to manually add"
                                            " exchanges configuration in your config.json without using any interface "
                                            "(ie the web interface that handle encryption automatically)",
                        action='store_true')
    parser.add_argument('-o', '--strategy_optimizer', help='Start Octobot strategy optimizer. This mode will make '
                                                           'octobot play backtesting scenarii located in '
                                                           'abstract_strategy_test.py with different timeframes, '
                                                           'evaluators and risk using the trading mode set in '
                                                           'config.json. This tool is useful to quickly test a '
                                                           'strategy and automatically find the best compatible '
                                                           'settings. Param is the name of the strategy class to '
                                                           'test. Example: -o TechnicalAnalysisStrategyEvaluator'
                                                           ' Warning: this process may take a long time.',
                        nargs='+')
    parser.set_defaults(func=start_octobot)

    # add sub commands
    subparsers = parser.add_subparsers(title="Other commands")

    # tentacles manager
    tentacles_parser = subparsers.add_parser("tentacles", help='Calls OctoBot tentacles manager.\n'
                                                               'Use "tentacles --help" to get the '
                                                               'tentacles manager help.')
    register_tentacles_manager_arguments(tentacles_parser)
    tentacles_parser.set_defaults(func=call_tentacles_manager)


def main(args=None):
    if not args:
        args = sys.argv[1:]
    parser = argparse.ArgumentParser(description='OctoBot')
    octobot_parser(parser)

    args = parser.parse_args(args)
    # call the appropriate command entry point
    args.func(args)



#  Drakkar-Software OctoBot
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
import asyncio
import pytest

from octobot_commons.tests.test_config import load_test_config
from octobot.commands import start_bot
from octobot.logger import init_logger
from octobot.octobot import OctoBot

# All test coroutines will be treated as marked.
pytestmark = pytest.mark.asyncio


async def test_run_bot():
    bot = OctoBot(load_test_config(), ignore_config=True)
    bot.task_manager.init_async_loop()
    await start_bot(bot, init_logger())
    await asyncio.sleep(10)
    bot.task_manager.stop_tasks()
